import React from 'react';
import { SelectionStateI, unSelect } from '@graphpolaris/shared/lib/data-access/store/interactionSlice';
import { useDispatch } from 'react-redux';
import { Button, EntityPill, useSelection } from '../../..';
import { BarPlot } from '@graphpolaris/shared/lib/components/charts/barplot';
import { Node, Edge, useGraphQueryResult } from '@graphpolaris/shared/lib/data-access';
import * as d3 from 'd3';

export const SelectionConfig = () => {
  const selection = useSelection();
  const dispatch = useDispatch();

  const result = useGraphQueryResult();

  if (!selection) return null;

  function getData(item: Node | Edge, attribute: string) {
    const data = result.nodes
      .filter(x => x.label == (item.attributes['labels'] as string[])[0])
      .map(x => x.attributes[attribute]);

    return data.map(x => ({
      category: "placeholder",
      count: parseFloat(x)
    }))

    // TODO: FIXME


    // const bins = d3.bin()(data);
    
    // return bins.map(bin => { 
    //   return {
    //     category: bin.x0,
    //     count: bin.length,
    //   }
    // });
  }

  function isNumber(item: Node | Edge, attribute: string) {
    return !Number.isNaN(parseFloat(item.attributes[attribute] as string));
  }

  function showPlot(item: Node | Edge, attribute: string) {

    if (isNumber(item, attribute)) {
      return true;
    }

    const data = result.nodes
      .filter(x => x.label == (item.attributes['labels'] as string[])[0])
      .map(x => x.attributes[attribute]);

    const countUnique = (new Set(data)).size;
    if (countUnique < 20) {
      return true;
    }

    return false;
  }

  return (
    <div className="border-b py-2 overflow-auto">
      <div className="flex justify-between items-center px-4 py-2">
        <span className="text-xs font-bold">Selection</span>
        <Button
          variantType="secondary"
          variant="ghost"
          size="xs"
          iconComponent="icon-[ic--baseline-delete]"
          onClick={() => {
            dispatch(unSelect());
          }}
        />
      </div>
      {selection.content.map((item, index) => (
        <React.Fragment key={index + 'id'}>
          <div className="flex justify-between items-center px-4 py-1 gap-1">
            <span className="text-xs font-normal">ID</span>
            <span className="text-xs">{item._id}</span>
          </div>
          <div key={index + 'label'} className="flex justify-between items-center px-4 py-1 gap-1">
            <span className="text-xs font-normal">Label</span>
            <EntityPill title={item.attributes['labels'] as string}></EntityPill>
          </div>
          <hr class="my-2" />
          <span className="text-xs font-bold  px-4">Attributes</span>
          {Object.entries(item.attributes).map(([key, value]) => {
            if (key === 'labels' || key === '_id') return null;
            return (
              <>
                { showPlot(item, key) && <div className="border-b">
                  { isNumber(item, key) ? <BarPlot
                        typeBarPlot="numerical"
                        numBins={20}
                        data={getData(item, key)}
                        marginPercentage={{ top: 0.1, right: 0, left: 0, bottom: 0 }}
                        className="h-[4rem] max-h-[4rem]"
                        name={key}
                        selection={selection}
                      />
                  :
                      <BarPlot
                        typeBarPlot="categorical"
                        numBins={20}
                        data={getData(item, key)}
                        marginPercentage={{ top: 0.1, right: 0, left: 0, bottom: 0 }}
                        className="h-[4rem] max-h-[4rem]"
                        maxBarsCount={100}
                        name={key}
                        selection={selection}
                      />
                  }
                </div>
                }
                <div key={index + key} className="flex justify-between items-center px-4 py-1 gap-1 border-b border-slate-400">
                  <span className="text-xs font-normal break-all max-w-[6rem]">{key}</span>
                  <span className="text-xs break-all">{value as string}</span>
                </div>
              </>
            );
          })}
        </React.Fragment>
      ))}
    </div>
  );
};
